#
# spec file for package dmd
#
# Copyright (c) 2018 SUSE LINUX GmbH, Nuernberg, Germany.
#
# All modifications and additions to the file contributed by third parties
# remain the property of their copyright owners, unless otherwise agreed
# upon. The license for this file, and modifications and additions to the
# file, is the same license as for the pristine package itself (unless the
# license for the pristine package is not an Open Source License, in which
# case the license is the MIT License). An "Open Source License" is a
# license that conforms to the Open Source Definition (Version 1.9)
# published by the Open Source Initiative.

# Please submit bugfixes or comments via http://bugs.opensuse.org/
#


%define bootstrap_version 2.078.0
%define sover	0_79
%if 0%{?sle_version} == 150000 && 0%{?is_opensuse}
%define auto_bootstrap 1
%else
%define auto_bootstrap 1
%endif
Name:           dmd
Version:        2.079.1
Release:        lp150.1.1
Summary:        D Programming Language 2.0
License:        BSL-1.0
Group:          Development/Languages/Other
URL:            http://dlang.org/
Source:         https://github.com/D-Programming-Language/dmd/archive/v%{version}.tar.gz#/%{name}-%{version}.tar.gz
Source1:        https://github.com/D-Programming-Language/druntime/archive/v%{version}.tar.gz#/druntime-%{version}.tar.gz
Source2:        https://github.com/D-Programming-Language/phobos/archive/v%{version}.tar.gz#/phobos-%{version}.tar.gz
Source8:        http://downloads.dlang.org/releases/2.x/%{bootstrap_version}/dmd.%{bootstrap_version}.linux.tar.xz
Source9:        dmd.conf
# PATCH-FIX-OPENSUSE dmd_use_tarball_bootstrap.diff So the tarball is not downloaded
Patch0:         dmd_use_tarball_bootstrap.diff
BuildRequires:  fdupes
BuildRequires:  gcc-c++
BuildRequires:  git
BuildRequires:  xz
Recommends:     phobos-devel
# Bootstrap is only possible with x86*
ExclusiveArch:  %{ix86} x86_64
%ifarch i586
#!BuildIgnore:  gcc-PIE
%endif
%if !%{auto_bootstrap}
BuildRequires:  dmd
BuildRequires:  phobos-devel-static
%endif

%description
The D programming language is an object-oriented, imperative,
multi-paradigm system programming language. It has type inference,
automatic memory management and syntactic sugar for common types,
bounds checking, design by contract features, and a concurrency-aware
type system.

%package 	-n 	libphobos2-%{sover}
Summary:        Standard library for the D language
Group:          System/Libraries

%description -n libphobos2-%{sover}
Phobos is the standard library for the D Programming Language.

This package contains the shared library needed to run programs compiled with
dmd.

%package 	-n 	phobos-devel
Summary:        Development files for the D standard library
Group:          Development/Libraries/Other
Requires:       libphobos2-%{sover} = %{version}

%description -n phobos-devel
Phobos is the standard library for the D Programming Language.

This package contains the includes needed to compile programs against Phobos.

%package 	-n 	phobos-devel-static
Summary:        Development files for the D standard library
Group:          Development/Libraries/Other
Requires:       phobos-devel = %{version}

%description -n phobos-devel-static
Phobos is the standard library for the D Programming Language.

This package contains the static library for static linking. You don't need
this, unless you link statically, which is highly discouraged.

%prep
%setup -q
%setup -q -T -D -a 1
%setup -q -T -D -a 2
%patch0 -p1
echo %{version} > phobos-%{version}/VERSION
cd %{_builddir}
ln -s dmd-%{version} dmd
ln -s druntime-%{version} druntime
ln -s phobos-%{version} phobos
mv dmd-%{version}/druntime-%{version} .
mv dmd-%{version}/phobos-%{version} .

%build
# dmd
cd ..
pushd dmd
	make %{?_smp_mflags} -f posix.mak \
	%if %{auto_bootstrap}
		AUTO_BOOTSTRAP=1 \
	%endif
		BUILD=release \
		PIC=1 \
		RELEASE=1
popd

# druntime
pushd druntime
	make %{?_smp_mflags} -f posix.mak \
		BUILD=release \
		DMD="../dmd/generated/linux/release/*/dmd" \
		PIC=1 \
		RELEASE=1
popd

# phobos
pushd phobos
	make %{?_smp_mflags} -f posix.mak \
		BUILD=release \
		DMD="../dmd/generated/linux/release/*/dmd" \
		PIC=1 \
		RELEASE=1
popd

%install
# install files manually since the install script distributed put files all over the place
# dmd
install -dm755 %{buildroot}%{_bindir}
install -Dm755 %{_builddir}/dmd/generated/linux/release/*/dmd %{buildroot}%{_bindir}/dmd

install -dm755 %{buildroot}%{_sysconfdir}
install -Dm644 %{_sourcedir}/dmd.conf %{buildroot}%{_sysconfdir}/dmd.conf

install -dm755 %{buildroot}%{_mandir}
cp -r %{_builddir}/dmd/docs/man/* %{buildroot}%{_mandir}/

install -dm755 %{buildroot}%{_datadir}/licenses/dmd
install -Dm644 %{_builddir}/dmd/LICENSE.txt %{buildroot}%{_datadir}/licenses/dmd/LICENSE.txt

install -dm755 %{buildroot}%{_datadir}/%{name}/samples
cp -r %{_builddir}/dmd/samples/* %{buildroot}%{_datadir}/dmd/samples/

# phobos
install -dm755 %{buildroot}%{_libdir}
cp -a $(find %{_builddir}/phobos/generated/linux/release/ \( -iname "*.a" -a \! -iname "*.so.a" \) -o \( -iname "*.so*" -a \! -iname "*.o" -a \! -iname "*.a" \) ) %{buildroot}%{_libdir}

install -dm755 %{buildroot}%{_includedir}/dlang/dmd
cp -r %{_builddir}/phobos/{*.d,etc,std} %{buildroot}%{_includedir}/dlang/dmd
cp -r %{_builddir}/druntime/import/* %{buildroot}%{_includedir}/dlang/dmd/

%fdupes %{buildroot}

/sbin/ldconfig -N %{buildroot}%{_libdir}

%post   -n libphobos2-%{sover} -p /sbin/ldconfig
%postun -n libphobos2-%{sover} -p /sbin/ldconfig

%files
%doc README.md
%license LICENSE.txt
%config(noreplace) %{_sysconfdir}/dmd.conf
%{_bindir}/dmd
%{_datadir}/dmd
%{_mandir}/man*/*

%files -n libphobos2-%{sover}
%{_libdir}/libphobos2.so.*

%files -n phobos-devel
%dir %{_includedir}/dlang
%dir %{_includedir}/dlang/dmd
%{_includedir}/dlang/dmd/*
%{_libdir}/libphobos2.so

%files -n phobos-devel-static
%{_libdir}/libphobos2.a

%changelog
* Mon Apr 16 2018 matthias.eliasson@gmail.com
- Update to 2.079.1
  - DMD Compiler regressions
  * [REG 2.073] [DIP1000] Error on bad interplay of 'auto ref' and 'return' attribute deduction.
  * DMD segmentation fault
  - DMD Compiler bugs
  * Enums are evaluated differently in global scope
  * compiler segfault if new on a union type with dip1000
  - DMD Compiler enhancements
  * Use of delete should be allowed without a deprecation in a deprecated scope
  - Phobos regressions
  * std.regex Captures opAssign returns void since v2.079.0
  * getSymbolsByUDA produces wrong result if one of the symbols having the UDA is a function
  * assigning a std.regex.Captures with 3 or more groups causes double free
  - Phobos bugs
  * [2.079] std.format ignores templated toString if another toString is not a template
  - Phobos enhancements
  * Document the type of std.stdio.stdin and friends
  - Druntime regressions
  * [Reg 2.079] src/rt/tracegc.d(43): Deprecation: The delete keyword has been deprecated
* Tue Apr 10 2018 matthias.eliasson@gmail.com
- Enable auto_bootstrap for Tumbleweed to get successful builds
- Run spec-cleaner
* Fri Mar 30 2018 guillaume.gardet@opensuse.org
- Build only for %%ix86 and x86_64 since other arch cannot be boostrapped
* Mon Mar  5 2018 matthias.eliasson@gmail.com
- Update to 2.079.0
  * Full changelog: https://dlang.org/changelog/2.079.0.html
- spec-cleaner was run
- use %%config(noreplace) for dmd.conf
  Make sure we don't replace if users modify this config.
* Thu Mar  1 2018 matthias.eliasson@gmail.com
- Enable auto_bootrap for Leap 15.0
- Add !buildignore gcc-PIE for i586 arch since it will not compile on i586 with PIE enabled
* Sun Feb 18 2018 matthias.eliasson@gmail.com
- Update to 2.078.3
  - DMD Compiler regressions
  * [REG2.078.2] Access violation when dmd tries to print highlighted code
  - DMD Compiler bugs
  * Switch skips over declaration issued for explicitly uninitialized variables
* Sun Feb 11 2018 matthias.eliasson@gmail.com
- Update to 2.078.2
  - DMD Compiler regressions
  * [REG 2.078] dmd can't generate 64-bit binaries on Windows 10 with VS 2015
  * [Reg 2.078] Building fails with VC 2015 Build Tools
  - DMD Compiler bugs
  * wrong code for int.min > 0
  - Druntime regressions
  * [Reg 2.078] comparison of arrays of associative arrays no longer compiles
* Wed Jan 24 2018 matthias.eliasson@gmail.com
- Update to 2.078.1
  - DMD Compiler regressions
  * [REG2.060] DMD hang in semantic3 on alias this
  - DMD Compiler bugs
  * undefined reference to __coverage
  * unittests get different names depending on how the files are passed to dmd
  * building with -m64 doesn't work with sc.ini from the zip distribution and VS2017
  - Phobos bugs
  * std.array.replace throws a range violation if the from range is longer than the array
  * Generic functions in std.math cannot be overloaded
  - Druntime regressions
  * module config is in file 'rt/config.d' which cannot be read
* Sat Jan 13 2018 matthias.eliasson@gmail.com
- Update to 2.078.0
- Update auto_bootstrap version to 2.078.0
- Remove bootrap option for ldc since we have working bootstraping with dmd
- Did some more clean-up in spec of build flags that was wrong and redundant
- Update default dmd.conf with more accurate includes
* Mon Jan  1 2018 matthias.eliasson@gmail.com
- Enable PIC and set Model=32 for i586 builds
- Disable auto_bootstrap
* Tue Dec 12 2017 matthias.eliasson@gmail.com
- Enable auto_bootstrap to make it build on i586.
* Sat Dec  9 2017 matthias.eliasson@gmail.com
- Update to 2.077.1
  * [REG2.068] compile-time string concatenation fails with CTFE and char[] literal involved
  * compiler segfault in DsymbolSemanticVisitor::visit(UnittestDeclaration*)
  * [REG 2.077.0] dmd 2.077.0 crashes when computing mangling symbol for simple program
  * [Reg 2.077] don't build libphobos2.a with PIC for i386
  * [Reg 2.075] Fibers guard page uses a lot more memory mappings
- Add "-fPIC" to DFLAGS in dmd.conf for 32bit so we can compile D programs for i586.
* Fri Nov 17 2017 joop.boonen@opensuse.org
- Added missing LICENSE.txt to docs
- Improved the spec file
* Sun Nov  5 2017 matthias.eliasson@gmail.com
- Update to 2.077.0
  * Mangled Symbols now back reference types and identifiers
  * Removed prelude assert for constructors & destructors
  * DMD builds are now reproducible
  * core.atomic.atomicLoad's returns types have changed for aggregate types that have indirections
  * Vectorized array operations are now templated
- Build with AUTO_BOOTSTRAP disabled
- Run spec-cleaner
* Thu Oct 26 2017 joop.boonen@opensuse.org
- Update to 2.076.1
  * fix Issue 16997 - Integral promotion rules not being followed for unary + - ~ operators
  * Mangled Symbols now back reference types and identifiers.
  * Removed prelude assert for constructors & destructors
  * DMD builds are now reproducible
- Build with AUTO_BOOTSTRAP
* Fri Jun  9 2017 jengelh@inai.de
- Rework description
* Fri May 26 2017 mailaender@opensuse.org
- Update to 2.073.2
- https://en.opensuse.org/SourceUrls
* Sat Mar 11 2017 matthias.eliasson@gmail.com
- Update to 2.073.1
* Thu Apr  7 2016 schuetzm@gmx.net
- Update to 2.071.0
  Compiler Changes
- Import access checks for fully qualified names were fixed.
- Protection for selective and renamed imports were fixed.
  Language Changes
- Imports no longer hide locals declared in outer scopes.
- Private symbols are no longer visible in other modules.
- Manifest constant can now be used for deprecation message.
  Library Changes
- A clear method has been added to associative arrays to remove all elements.
- The GC now uses a spinlock instead of a recursive mutex.
- Calls to GC.free are now ignored during finalization instead of throwing an InvalidMemoryOperationError, see Bugzilla 15353.
- GC.addRoot and GC.addRange now use a separate lock.
- GCAllocator.goodAllocSize was added.
- High-level API of std.net.curl now uses Appender for received content. Which generally makes all calls slightly faster. Up to 200ms for large amounts of data.
- Repeat was added to obtain a repeating AliasSeq consisting of template arguments.
- fold was added as an alternative to reduce with argument order swapped.
- Added nextPow2 and truncPow2 to std.math.
- bsf, bsr, and popcnt now work with ulongs on 32 bit systems.
- topN is now 5%% - 7%% faster.
- Unary overloads of startsWith and endsWith were added.
- maxCount and maxPos were added.
- Range support for the convenience wrapper redBlackTree was added.
  See the full changelog here:
  http://dlang.org/changelog/2.071.0.html
* Fri Mar  4 2016 schuetzm@gmx.net
- Update to 2.070.2
  This is a bugfix release, please refer to the changelog for details:
  http://dlang.org/changelog/2.070.2.html
* Sun Feb 28 2016 schuetzm@gmx.net
- Update to 2.070.1
  This is a bugfix release, see the changelog for details:
  http://dlang.org/changelog/2.070.1.html
* Thu Jan 28 2016 schuetzm@gmx.net
  Update to 2.070.0
  Language Changes
- Add syntactic support to make an alias to a function literal.
  Library Changes
- The package std.experimental.ndslice was added. It is also available as part of the Mir library.
- The core.sys.windows package has been greatly expanded.
- TypeInfo.init has been renamed to TypeInfo.initializer.
- std.json encodes string control characters.
- Default LogLevel of FileLogger was changed to LogLevel.all.
- Access to the internal ptr and len fields of BitArray was deprecated.
  See the full changelog here:
  http://dlang.org/changelog/2.070.0.html
* Thu Dec  3 2015 schuetzm@gmx.net
  Update to 2.069.2
  This is a bugfix release, see the changelog for details:
  http://dlang.org/changelog/2.069.2.html
* Thu Nov  5 2015 schuetzm@gmx.net
  Update to 2.069
  This is the first release with a self-hosted dmd compiler and comes
  with even more rangified phobos functions,
  std.experimental.allocator, and many other improvements.
  See the changelog for more details.
  http://dlang.org/changelog/2.069.0.html
* Thu Sep 24 2015 schuetzm@gmx.net
- update 2.068.2
- This point release fixes a few regressions 2.068.1, see the
  changelog for more details:
  http://dlang.org/changelog/2.068.2.html
* Mon Sep  7 2015 schuetzm@gmx.net
- update to 2.068.1
- Fixes bugs and regressions in 2.068.0
* Mon Aug 10 2015 schuetzm@gmx.net
- update to 2.068.0
  Notable changes:
- many rangified phobos functions
- 2 new GC profilers
- a new AA implementation
- countless further improvements and fixes
* Sun Jun  7 2015 schuetzm@gmx.net
- update to 2.067.1
* Wed Mar 25 2015 schuetzm@gmx.net
- update to 2.067.0
* Tue Nov 25 2014 pastas4@gmail.com
- add a valid dmd.conf file (from upstream)
- recommend phobos-devel
* Sun Nov 16 2014 mailaender@opensuse.org
- use https://en.opensuse.org/SourceUrls
- new project URL
- split libphobos as discussed in bos#905648
- add Artistic license
- deprecate dmd-fix-build.patch
- deprecate dmd-rpmlintrc
* Sun Nov 16 2014 pastas4@gmail.com
- update to 2.066.1
* Sun Jun 30 2013 seife+obs@b1-systems.com
- fix license tag, Boost -> BSL-1.0
* Sat May 19 2012 opensuse@dstoecker.de
- add config file for include paths
- update to 2.059
* Fri Mar 23 2012 jengelh@medozas.de
- Parallel build with %%_smp_mflags; strip redundant sections/tags
- SPDX license field
* Thu May 19 2011 prusnak@opensuse.org
- created package (version 2.053)
